@@ -4,8 +4,9 @@
 #include "decoders.h"
 #include "audiobuffer.h"
 #include "sound.h"
+#ifdef USE_FAAD
 #include "m4a.h"
-#include "utils/utils.h"
+#endif
 
 #include <stdatomic.h>
 
@@ -105,12 +106,11 @@ void soundResumePlayback(void)
 {
         // If this was unpaused with no song loaded
 
-        AudioData *audioData = getAudioData();
         AppState *state = getAppState();
 
-        if (audioData->restart)
+        if (audioData.restart)
         {
-                audioData->endOfListReached = false;
+                audioData.endOfListReached = false;
         }
 
         if (!ma_device_is_started(&device))
